/**
 * Profile.class.js v0.2 - Load/Save INI file class
 *
 *	vim: set ft=javascript:
 *	Author: M.Satoh
 *	Last Change: 16-Nov-2009.
 */

(function(){
	var _sections;

	this.load = function(path) {
		_sections = {};
		var fs = WSH.require("FileSystem");
		if (!fs.isFileExists(path)) {
			return;
		}

		var section = "";
		var f = fs.openTextFile(path);
		while (!f.AtEndOfStream) {
			// 1sǂݍ
			var line = f.ReadLine();
			line = line.replace(/[\r\n]/g, "");
			if (line == "" || line.match(/^\s*;/)) {
				// RgsƋs̓XLbv
				continue;
			}
			if (line.match(/^\[(.*)\]$/)) {	// ZNV
				section = RegExp.$1;
				if (section in _sections != true) {
					_sections[section] = {};
				}
			}
			else if (line.match(/^([^=]+)=(.*)$/)) {	// Key=Value
				if (section in _sections) {
					_sections[section][RegExp.$1] = RegExp.$2;
				}
			}
		}
		f.Close();
	};

	this.save = function(path) {
		var fs = WSH.require("FileSystem");
		var lines = new Array();
		var f;
		if (fs.isFileExists(path)) {	// t@Cꍇ
			f = fs.openTextFile(path);
			var section = "";
			while (!f.AtEndOfStream) {
				// 1sǂݍ
				var line = f.ReadLine();
				line = line.replace(/[\r\n]/g, "");
				if (line == "" || line.match(/^\s*;/)) {
					// CommentƋ󔒍s͉HȂ
				}
				else if (line.match(/^\[(.*)\]$/)) {	// Section̊Jn
					if (section in _sections) {
						// ݂SectionŖ̂̂
						foreach(_sections[section], function(k, v) {
							lines.push(k + "=" + v);
						});
						lines.push("");
						delete _sections[section];
					}
					section = RegExp.$1;
				}
				else if (line.match(/^([^=]+)=(.*)$/)) {	// Key=Value
					if (section in _sections) {
						if (RegExp.$1 in _sections[section]) {
							// YKey΁A̒lɕύX
							line = RegExp.$1 + "=";
							line += _sections[section][RegExp.$1];
							delete _sections[section][RegExp.$1];
						}
						else {
							// Ȃ΍폜
							line = "";
						}
					}
				}
				lines.push(line);
			}
			f.Close();
			if (section in _sections) {
				// ݂SectionŖ̂̂
				foreach(_sections[section], function(k, v) {
					lines.push(k + "=" + v);
				});
				lines.push("");
				delete _sections[section];
			}
		}
		foreach(_sections, function(k1, v1) {
			lines.push("");
			lines.push("[" + k1 + "]");
			var length = lines.length;
			foreach(v1, function(k2, v2) {
				lines.push(k2 + "=" + v2);
			});
			if (lines.length == length) {
				lines.length = length - 2;
			}
		});
		// t@C֏o
		f = fs.createTextFile(path);
		var prevLine = "";
		foreach(lines, function(k, v) {
			if (v != "" || prevLine != "") {
				f.WriteLine(v);
				prevLine = v;
			}
		});
		f.Close();
		// ēǍ
		this.load(path);
	};

	this.get = function(section, key, defaultValue) {
		if (section in _sections != true) {
			_sections[section] = {};
		}
		if (arguments.length == 1) {
			return _sections[section];
		}
		else if (key in _sections[section]) {
			return _sections[section][key];
		}
		else if (arguments.length == 3) {
			_sections[section][key] = defaultValue;
			return defaultValue;
		}
		return null;
	};

	this.set = function(section, key, value) {
		if (section in _sections != true) {
			_sections[section] = {};
		}
		if (arguments.length == 3) {
			_sections[section][key] = value;
		}
		else {
			foreach(key, function(k, v) {
				_sections[section][k] = v;
			});
		}
	};

	this.remove = function(section, key) {
		if (section in _sections) {
			if (arguments.length == 2) {
				if (key in _sections[section]) {
					delete _sections[section][key];
				}
			}
			else {
				delete _sections[section];
			}
		}
	};

}).apply(WSH.Profile);

