/**
 * stdlib.js v0.4 - Javascript Standard Library
 *
 *	vim: set ft=javascript:
 *	Author: M.Satoh
 *	Last Change: 16-Nov-2009.
 */

/**
 * NXIuWFNg̊NX
 *	<Usage>
 *		var NewClass = Class.create();
 *		(function() {
 *			this.initialize = function(a1, a2, ...) {
 *				this.a1 = a1;
 *				[other codes]
 *			};
 *			this.func = function(...) { ... };
 *		}).apply(NewClass.prototype);
 *
 *		var obj = new NewClass(a1, a2, ...);
 *		obj.func(...);
 */
var Class = {};
(function() {
	this.create = function() {
		return function() {
			// hNX̃RXgN^Ăяo
			this.initialize.apply(this, arguments);
		};
	};
}).apply(Class);

/**
 * XNvg̊NX
 *	<Usage>
 *		var NewScript = Script.create();
 *		(function() {
 *			this.initialize = function(a1, a2, ...) {
 *				this.a1 = a1;
 *					:
 *					:
 *			};
 *			this.main = function(argv) {
 *				hoge;
 *			};
 *			this.OnLoad = function(evt) {
 *				fuga;
 *			};
 *		}).apply(NewScript.prototype);
 *		new NewScript(a1, a2, ...); // Iinitialize(), main()Ă΂
 */
var Script = {};
(function(){
 	this.create = function() {
		return function() {
			// hNX̃RXgN^Ăяo
			this.initialize.apply(this, arguments);

			if (typeof(WScript) != "undefined") {
				// WSH̏ꍇAmain()Ăяo
				var argv = {};
				var re = new RegExp("(/[^:]+):(.*)$");
				foreach(WScript.Arguments, function(i, it) {
					if (it.match(re)) {
						argv[RegExp.$1] = RegExp.$2;
					}
					else {
						argv[i] = it;
					}
				});
				this.main(argv);
			}
			else if (typeof(window) != "undefined") {
				// HTȀꍇAwindow.onload Cxgŏ
				var self = this;
				window.attachEvent("onload", function() {
					// ReLXgj[\
					if (typeof(self.contextmenu) == "boolean" &&
						self.contextmenu === false)
					{
						document.attachEvent("oncontextmenu", function() {
							return false;
						});
					}
					foreach(self, function(key, func) {
						if (isFunction(func) == false) {
							return false; // continue
						}
						var action;
						if (key.match(/^Document_(On.+)$/i)) {
							action = RegExp.$1.toLowerCase();
							document.attachEvent(action, func);
						}
						else if (key.match(/^(.+)_(On.+)$/i)) {
							// DOM Node ɃCxgnhݒ
							var elem = $(RegExp.$1);
							if (elem) {
								action = RegExp.$2.toLowerCase();
								elem.attachEvent(action, func);
							}
						}
						else if (key.match(/^On.+$/i)) {
							// window ɃCxgnhݒ
							action = key.toLowerCase();
							if (action == "onload") {
								// OnLoad()͒ڌĂ
								func.apply(self);
							}
							else {
								window.attachEvent(action, func);
							}
						}
					});
				});
			}
		};
	};
}).apply(Script);

/**
 * O[oIuWFNg̊g
 */
/**
 * ^Fstring
 */
function isString(obj) {
	return (typeof(obj) == "string" || obj instanceof String);
}

/**
 * ^Ffunction
 */
function isFunction(obj) {
	return (typeof(obj) == "function");
}

/**
 * ^Farray
 */
function isArray(obj) {
	return (typeof(obj) == "object" && obj instanceof Array);
}

/**
 * ^Fnumber
 */
function isNumber(obj) {
	return (typeof(obj) == "number");
}

/**
 * foreach
 * 	@param obj [variant] Ώ
 * 	@param iterator [function] (key, value)֐
 */
function foreach(obj, iterator) {
	var i;
	if (isArray(obj) || isString(obj)) {
		// z܂͕
		for (i = 0; i < obj.length; i++) {
			if (iterator(i, obj[i]) === true) {
				return;
			}
		}
	}
	else if (typeof(obj.Count) == "unknown" && typeof(obj.Item) == "unknown") {
		// WshArguments
		for (i = 0; i < obj.Count(); i++) {
			if (iterator(i, obj.Item(i)) === true) {
				return;
			}
		}
	}
	else if (obj.length && obj.item) {
		// (DOM)NodeList
		for (i = 0; i < obj.length; i++) {
			if (iterator(i, obj.item(i)) === true) {
				return;
			}
		}
	}
	else if (isNumber(obj.Count) && typeof(obj.toString) == "undefined") {
		// (WMI)Collection???
		var objEnum = new Enumerator(obj);
		for (i = 0; !objEnum.atEnd(); i++, objEnum.moveNext()) {
			if (iterator(i, objEnum.item()) === true) {
				return;
			}
		}
	}
	else if (isNumber(obj.length) && isFunction(obj.callee)) {
		// arguments
		for (i = 0; i < obj.length; i++) {
			if (iterator(i, obj[i]) === true) {
				return;
			}
		}
	}
	else {
		// other, Object
		for (var key in obj) {
			if (iterator(key, obj[key]) === true) {
				return;
			}
		}
	}
}

/**
 * wscript.exeŎsĂtrueԂ
 */
var isWScript = function() {
	if (typeof(WScript) == "object") {
		return WScript.FullName.match(/wscript\.exe$/i) ? true : false;
	}
	return false;
}();

/**
 * cscript.exeŎsĂtrueԂ
 */
var isCScript = function() {
	if (typeof(WScript) == "object") {
		return WScript.FullName.match(/cscript\.exe$/i) ? true : false;
	}
	return false;
}();

/**
 * wscript/cscript؂ւ
 */
function SwitchScriptHost() {
	var cmd = "";
	if (isWScript) {
		cmd = "cmd.exe /k cscript //Nologo ";
	}
	else if (isCScript) {
		cmd = "wscript.exe ";
	}
	else {
		throw new Error("WSHOSwitchScriptHost()Ă΂܂B");
	}

	cmd += WScript.ScriptFullName.qq();
	foreach(WScript.Arguments, function(i, value) {
		if (value.match(/ /)) {
			value = value.qq();
		}
		cmd += " " + value;
	});

	WSH.require("Shell").Run(cmd, WSH.WindowStyle.Normal);
	WScript.Quit();
}

/**
 * R\[o͂܂̓bZ[W{bNX\
 */
function echo(str) {
	if (typeof(WScript) === "object") {
		WScript.Echo(str);
	}
	else if (typeof(window) === "object") {
		alert(str);
	}
}

/**
 * StringIuWFNgg
 */
(function() {
	/**
	 * s -> <br/>
	 */
	String.prototype.nl2br = function() {
		var str = this;
		str = str.replace(/\r\n/g, "<br/>");
		str = str.replace(/(\r|\n)/g, "<br/>");
		return str;
	};

	/**
	 * HTMLGXP[v
	 */
	String.prototype.htmlspecialchars = function() {
		var str = this;
		str = str.replace(/&/g, "&amp;");
		str = str.replace(/"/g, "&quot;");
		str = str.replace(/'/g, "&#039;");
		str = str.replace(/</g, "&lt;");
		str = str.replace(/>/g, "&gt;");
		return str ;
	};

	/**
	 * VONH[gň͂
	 */
	String.prototype.q = function() {
		return "'" + this + "'";
	};

	/**
	 * _uNH[gň͂
	 */
	String.prototype.qq = function() {
		return '"' + this + '"';
	};
})();

/**
 * MathIuWFNgg
 */
(function() {
	/**
	 * ͈͓̗𐮐Ő
	 */
	Math.rand = function(max, min) {
		if (arguments.length == 0) {
			return Math.round(Math.random()); // 0 or 1
		}
		if (arguments.length == 1) {
			min = 0;
		}
		return Math.floor((max - min + 1) * Math.random() + min);
	};
})();

/**
 * windowIuWFNgg
 */
(function() {
	/**
	 * getElementById()̊g
	 */
	function _getElementById(id) {
		return document.getElementById(id);
	}

	/**
	 * getElementsByTagName()̊g
	 */
	function _getElementsByTagName(name, root) {
		if (!root) {
			root = document;
		}
		else if (isString(root)) {
			root = _getElementById(root);
		}
		return root.getElementsByTagName(name);
	}

	if (typeof(window) == "object") {
		window.$ = _getElementById;
		window.$$ = _getElementsByTagName;
	}
})();

/**
 * WSH IuWFNg
 */
var WSH = WSH || {};
(function() {
	/**
	 * W[̃C|[g
	 */
	this.require = function(module) {
		if (this[module] && this[module].length > 0) {
			return this[module];
		}
		var objFso = new ActiveXObject("Scripting.FileSystemObject");
		var fname = module + ".class.js";
		var dir = [".", ".\\js", "..\js"];
		var path = "";
		foreach(dir, function(key, value) {
			if (objFso.FileExists(value + "\\" + fname)) {
				path = value + "\\" + fname;
				return true;
			}
		});
		if (path) {
			var objFile = objFso.OpenTextFile(path);
			var code = objFile.ReadAll();
			objFile.Close();
			this[module] = {};
			eval(code);
			return this[module];
		}
		return null;
	};
}).apply(WSH);

